import * as t from '@babel/types';
import { File } from '@babel/types';
import { GeneratorOptions } from '@babel/generator';
import { Node, VText, VLiteral, ESLintLiteral, ESLintProgram as ESLintProgram$1 } from 'vue-eslint-parser-private/ast/nodes';
import { ParserOptions as ParserOptions$1 } from '@babel/parser';
import { ESLintProgram, ESLintTemplateLiteral } from 'vue-eslint-parser-private/ast';
import { Linter } from 'eslint';
import { TraverseOptions } from '@babel/traverse';

type ParserType = 'js' | 'jsx' | 'ts' | 'tsx' | 'vue';
interface ParserOptions {
    vue?: Linter.ParserOptions;
    babel?: ParserOptions$1;
}
interface Props {
    content: string;
    type: ParserType;
    parserOptions?: ParserOptions;
    debug?: boolean;
}
declare class Parser implements Props {
    readonly origin: string;
    readonly content: string;
    readonly type: ParserType;
    readonly ast: File;
    readonly parserOptions: Required<ParserOptions>;
    readonly astVueTemplate: ESLintProgram;
    readonly debug?: boolean;
    readonly vueScriptContent?: string;
    ignoreLine: Record<number, boolean>;
    ignoreFile?: {
        code: string;
        stack: Record<string, string>[];
    };
    constructor(props: Props);
    get isVue(): boolean;
    get isVueSetup(): boolean;
    get isReact(): boolean;
    get isJS(): boolean;
    get realContent(): string;
    private _parserVue;
    private _parserByBabel;
    private _comment;
}

type parser_d_ParserType = ParserType;
type parser_d_ParserOptions = ParserOptions;
type parser_d_Props = Props;
declare namespace parser_d {
  export {
    Parser as default,
    parser_d_ParserType as ParserType,
    parser_d_ParserOptions as ParserOptions,
    parser_d_Props as Props,
  };
}

type ArrayElement<ArrayType extends readonly unknown[]> = ArrayType extends readonly (infer ElementType)[] ? ElementType : never;
interface Options {
    readonly identifier?: string;
    ruleKey?: (node: t.Node | Node, path?: string, value?: string) => string | number;
    parserOptions?: ParserOptions;
    generatorOptions?: GeneratorOptions;
    react?: {
        i18n?: string;
        i18nLibName?: string;
        i18nHookName?: string;
        i18nFunctionName?: string;
    };
    path?: string;
    experimental?: boolean;
    customTraverse?: TraverseOptions;
    debug?: boolean;
}
interface Config extends Options {
    type: ParserType;
}
interface CommandArgs {
    code: string;
    name: string;
    dir: string;
    type: ParserType;
    stack: string;
    config: string;
    debug: boolean;
    experimental: boolean;
    feedback: boolean;
}

declare class VueHelpers {
    private map;
    readonly parser: Parser;
    readonly options: Options | undefined;
    private identifier;
    private stack;
    private content;
    constructor(parser: Parser, options?: Options);
    _renderKey(ast: Node, value: string): string | number;
    _generate(): TransformRet;
    _traverseTemplateBody(ast: VText | VLiteral | ESLintLiteral): void;
    _traverseTemplateLiteral(ast: ESLintTemplateLiteral): void;
    _traverse(ast: ESLintProgram$1, keys: string[]): void;
    _transform(): void;
    generate(scriptRet: TransformRet): TransformRet;
}

interface TransformRet {
    code: string;
    stack: Record<string, string>[];
    ast?: t.File;
}
declare class Transform {
    readonly parser: Parser;
    readonly options: Options | undefined;
    readonly stack: Record<string, string>[];
    private identifier;
    VueHelpers: VueHelpers;
    constructor(parser: Parser, options?: Options);
    _key(node: t.Node, value: string): string | number;
    _StringFunction(node: t.StringLiteral, identifier: t.Identifier | null): t.CallExpression;
    _TemplateFunction(node: t.TemplateLiteral, identifier: t.Identifier | null): t.CallExpression;
    _JSXTextFunction(node: t.JSXText, identifier: t.Identifier | null): t.JSXExpressionContainer;
    _JSXAttributeFunction(node: t.JSXAttribute, identifier: t.Identifier | null): t.JSXAttribute;
    _transform(): t.File;
    _generate(ast: t.File, options: GeneratorOptions): {
        code: string;
        stack: Record<string, string>[];
        ast: t.File;
    };
    render(options?: GeneratorOptions): TransformRet;
}

type transform_d_TransformRet = TransformRet;
declare namespace transform_d {
  export {
    Transform as default,
    transform_d_TransformRet as TransformRet,
  };
}

declare function transformCode(code: string, config: Config): TransformRet;

export { ArrayElement, CommandArgs, Config, Options, parser_d as Parser, transform_d as Transform, transformCode };
